import pylsl


def show(args):
	stream_name = args.name

	infos = pylsl.resolve_streams(args.timeout)
	names = [i.name() for i in infos]

	if stream_name not in names: 
		print('Stream not found.')
		return

	stream_info = infos[names.index(stream_name)]
	inlet = pylsl.StreamInlet(stream_info)

	print(inlet.info().as_xml())



import pylsl
from .utils import STREAM_INFO_FIELDS, infos_to_dict
from .list import print_infos_line


def find(args): 

	props = {}
	for field in STREAM_INFO_FIELDS:
		try: 
			prop = getattr(args, field)
		except KeyError: 
			pass

		if prop is not None:
			props[field] = prop

	if len(props) == 0: 
		return False

	infos = pylsl.resolve_streams(args.timeout)
	
	found = []
	for info in infos:
		gen_is_ok = (str(getattr(info, field)()) == props[field] for field in props.keys()) 

		if all(gen_is_ok):
			found.append(info)

	if len(found) > 0: 
		infdict = infos_to_dict(found)
		print_infos_line(infdict)

	else: 
		print("No stream found.")


import pylsl
import time 
from datetime import datetime


def echo(args): 
	stream_name = args.name

	infos = pylsl.resolve_streams(args.timeout)
	names = [i.name() for i in infos]

	if stream_name not in names: 
		print('Stream not found.')
		return

	stream_info = infos[names.index(stream_name)]

	if stream_info.nominal_srate() != pylsl.IRREGULAR_RATE:
		delay = 0.9 / stream_info.nominal_srate() 
	else: 
		delay = 0.1

	inlet = pylsl.StreamInlet(stream_info)
	
	running = True
	while running:
		try: 
			res = inlet.pull_sample(0)
			if res[0] != None:
				sample, timestamp = res
				print('{time:<10} {sample}'.format(
					time=datetime.fromtimestamp(float(timestamp)).strftime('%H:%M:%S.%f'), 
					sample=sample))
			time.sleep(delay)
		except KeyboardInterrupt:
			running = False
			
	


import pylsl
from datetime import datetime
from .utils import *


def print_infos(infos, fields=STREAM_INFO_FIELDS): 
	fields = {fields} if isinstance(fields, str) else fields

	for field in fields:
		if field not in STREAM_INFO_FIELDS:
			raise ValueError(f"Unkown field: {field}")

	for info in infos: 
		print(info.name())
		for field in fields:
			print(f'  {field}: {getattr(info, field)()}')


INFO_LINE_FIELDS = [
	'type', 'channel_count', 'nominal_srate', 'created_at', 'name'
]
INFO_LINE_FORMAT = {
	'created_at': lambda s: datetime.fromtimestamp(float(s)).strftime('%H:%M:%S')
}
INFO_LINE_HEADERS = {
	'type':'type', 'channel_count':'chans', 'nominal_srate': 'rate', 'created_at': 'created', 'name': 'name'
}

def format_field(key, value):
	return '%s'%value if key not in INFO_LINE_FORMAT else INFO_LINE_FORMAT[key](value)

def print_infos_line(infos: dict):
	infos = [
		{k:format_field(k, v) for k, v in info.items() if k in INFO_LINE_FIELDS}
		for info in infos 
	]

	lengths = {k: len(INFO_LINE_HEADERS[k]) for k in infos[0].keys()}
	for i in infos:
		lengths.update(
			{
				k: max(lengths[k], len(i[k])) 
				for k in i.keys()}
		)

	line_format = ''
	for k in INFO_LINE_FIELDS:
		line_format = line_format + '{%s:<%d}  '%(k, lengths[k])

	print(line_format.format(**{k:INFO_LINE_HEADERS[k] for k in INFO_LINE_FIELDS}))
	for i in infos:
		print(line_format.format(**i))



def list(args): 
	if args.all:
		fields = STREAM_INFO_FIELDS
	else: 
		fields = [f for f in STREAM_INFO_FIELDS if getattr(args, f)]

	infos = pylsl.resolve_streams(float(args.timeout))
	if len(infos) == 0:
		return 

	if args.list:
		infodict = infos_to_dict(infos)
		print_infos_line(infodict)

	else: 
		print_infos(infos, fields)




import pylsl


STREAM_INFO_FIELDS = (
	'name',
	'type', 
	'source_id', 
	'channel_count', 
	'channel_format', 
	'nominal_srate', 
	'hostname', 
	'uid', 
	'version', 
	'session_id', 
	'created_at'
)

def infos_to_dict(infos): 
	if not hasattr(infos, '__iter__'):
		infos = [infos]
	return [{
		f: getattr(info, f)() 
		for f in STREAM_INFO_FIELDS
	} for info in infos]

def filter_stream_fields(infos, **fields): 
	for info in infos:
		pass


from .utils import *
from .list import list
from .echo import echo
from .show import show
from .find import find
import argparse



def complete(args): 
	import subprocess
	import os
	path = os.path.dirname(os.path.abspath(__file__))
	os.chdir(path)

	if args.zsh:
		_args = ['zsh']
	elif args.bash:
		_args = ['bash']
	else: 
		_args = []
	subprocess.call([os.path.join(path, 'extra/lsl_complete_script.sh')] + _args, env=os.environ.copy())
	

def main():
	parser = argparse.ArgumentParser()

	subparser = parser.add_subparsers(dest='command')

	complete_parser = subparser.add_parser('complete')
	complete_parser_group = complete_parser.add_mutually_exclusive_group()
	complete_parser_group.add_argument('--zsh', '-z', action='store_true', dest='zsh')
	complete_parser_group.add_argument('--bash', '-b', action='store_true', dest='bash')

	complete_parser.set_defaults(func=complete)

	list_parser = subparser.add_parser('list')
	list_parser.add_argument('--all', '-a', dest='all', action='store_true')
	list_parser.add_argument('--list', '-l', dest='list', action='store_true')
	list_parser.add_argument('--timeout', dest='timeout', type=float, default=0.05)

	for field in STREAM_INFO_FIELDS:
		list_parser.add_argument(f'--{field}', dest=field, action='store_true', help=f"Display stream {field}")
	
	list_parser.set_defaults(func=list)

	echo_parser = subparser.add_parser('echo', description="Print to screen messages from a stream.")
	echo_parser.add_argument('name', help='Stream name')
	echo_parser.add_argument('--timeout', dest='timeout', type=float, default=0.05)

	echo_parser.set_defaults(func=echo)

	show_parser = subparser.add_parser('show')
	show_parser.add_argument('name', help='Stream name')
	show_parser.add_argument('--timeout', dest='timeout', type=float, default=0.05)

	show_parser.set_defaults(func=show)

	find_parser = subparser.add_parser('find')
	for field in STREAM_INFO_FIELDS:
		find_parser.add_argument(f'--{field}', dest=field, type=str, help=f"Display stream {field}")
	find_parser.add_argument('--timeout', dest='timeout', type=float, default=0.05)
	find_parser.set_defaults(func=find)

	args = parser.parse_args()
	if args.command is None:
		parser.print_help()
	else: 
		res = args.func(args)
		if res is False:
			parser.print_help()






from setuptools import setup
from pathlib import Path

this_dir = Path(__file__).parent
long_description = (this_dir / "README.md").read_text()


setup(
	name="lsl-cli", 
	version='0.1.0', 
	description="Command-line tools for the Lab Streaming Layer.",
	url="https://github.com/yop0/lsl-cli", 
	author="Johan Medrano",
	author_email="johan.medrano653@gmail.com",
    long_description=long_description,
    long_description_content_type='text/markdown',
	license="MIT",
	entry_points = {
		'console_scripts': [
			'lsl=lsl_cli.lsl:main'
		]
	}, 
	packages=['lsl_cli'],
	install_requires = [
		'pylsl'
	],
	package_data = {
		'lsl_cli': ['extra/lsl-completion.*', 'extra/lsl_complete_script.sh']
	}, 
	classifiers = [
        'Development Status :: 1 - Planning',
	]
) 
